<?php
/**
 * Helper functions
 *
 * @package     RCP\MailChimpPro\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Get MailChimp subscription lists
 *
 * @param bool $cache Whether or not to get cached results.
 *
 * @since       1.0.0
 * @return      array|false $lists array The list of available lists or false on failure.
 */
function rcp_mailchimp_pro_get_lists( $cache = true ) {

	$lists = get_transient( 'rcp_mailchimp_pro_lists' );

	// Return cached lists if it exists.
	if ( $cache && ! empty( $lists ) ) {
		return $lists;
	}

	$settings = get_option( 'rcp_mailchimp_pro_settings' );

	$api_key = ! empty( $settings['api_key'] ) ? trim( $settings['api_key'] ) : false;

	// No API key - bail.
	if ( empty( $api_key ) ) {
		return false;
	}

	$data_center = explode( '-', $api_key );
	$data_center = $data_center[1];

	$request_url  = 'https://' . urlencode( $data_center ) . '.api.mailchimp.com/3.0/lists/?count=150';
	$request_args = array(
		'headers' => array(
			'Authorization' => 'Basic ' . base64_encode( 'user:' . $api_key )
		)
	);

	$response = wp_remote_get( $request_url, $request_args );

	if ( is_wp_error( $response ) || 200 != wp_remote_retrieve_response_code( $response ) ) {
		return false;
	}

	$data = json_decode( wp_remote_retrieve_body( $response ), true );

	if ( empty( $data['lists'] ) || ! is_array( $data['lists'] ) ) {
		return array();
	}

	$lists = array();

	foreach ( $data['lists'] as $list_info ) {
		$lists[ $list_info['id'] ] = $list_info['name'];
	}

	set_transient( 'rcp_mailchimp_pro_lists', $lists, DAY_IN_SECONDS );

	return $lists;

}


/**
 * Check if we should show the subscription field
 *
 * @since       1.0.0
 * @param       int $level A specific level to check
 * @return      bool $show True to show, false otherwise
 */
function rcp_mailchimp_pro_show_checkbox() {
	$settings = get_option( 'rcp_mailchimp_pro_settings' );
	$lists    = get_option( 'rcp_mailchimp_pro_subscription_lists' );
	$show     = false;

	if( ! empty( $settings['api_key'] ) ) {
		if( ! empty( $settings['saved_list'] ) ) {
			$show = true;
		}

		if( is_array( $lists ) ) {
			$show = true;
		}
	}

	return $show;
}


/**
 * Get interest groups for MailChimp lists
 *
 * @since       1.2.3
 * @param       string $list  ID of the list to get the groups for.
 * @param       bool   $cache Whether or not to fetch cached results.
 * @return      object|false Response object from MailChimp or false on failure.
 */
function rcp_mailchimp_pro_get_groups( $list, $cache = true ) {

	$groups = get_transient( 'rcp_mailchimp_pro_list_groups_' . $list );

	// Return cached groups if it exists.
	if ( $cache && ! empty( $groups ) ) {
		return $groups;
	}

	$settings = get_option( 'rcp_mailchimp_pro_settings' );

	$api_key = ! empty( $settings['api_key'] ) ? trim( $settings['api_key'] ) : false;

	// No API key - bail.
	if ( empty( $api_key ) ) {
		return false;
	}

	$data_center = explode( '-', $api_key );
	$data_center = $data_center[1];

	$request_url  = 'https://' . urlencode( $data_center ) . '.api.mailchimp.com/3.0/lists/' . urlencode( $list ) . '/interest-categories/?count=100';
	$request_args = array(
		'headers' => array(
			'Authorization' => 'Basic ' . base64_encode( 'user:' . $api_key )
		)
	);

	$response = wp_remote_get( $request_url, $request_args );

	if ( is_wp_error( $response ) || 200 != wp_remote_retrieve_response_code( $response ) ) {
		return false;
	}

	$response_body = json_decode( wp_remote_retrieve_body( $response ) );

	if ( is_object( $response_body ) ) {
		set_transient( 'rcp_mailchimp_pro_list_groups_' . $list, $response_body, DAY_IN_SECONDS );
	}

	return $response_body;
}

/**
 * Get interests in a given interest category
 *
 * @param string $list_id              ID of the list to get interests from.
 * @param string $interest_category_id ID of the interest category to get interests from.
 *
 * @since 1.3.3
 * @return array|false Array of interests (`id` => `name`) or false on failure.
 */
function rcp_mailchimp_pro_get_interests( $list_id, $interest_category_id ) {

	$settings = get_option( 'rcp_mailchimp_pro_settings' );

	$api_key = ! empty( $settings['api_key'] ) ? trim( $settings['api_key'] ) : false;

	// No API key - bail.
	if ( empty( $api_key ) ) {
		return false;
	}

	$data_center = explode( '-', $api_key );
	$data_center = $data_center[1];

	$request_url  = 'https://' . urlencode( $data_center ) . '.api.mailchimp.com/3.0/lists/' . urlencode( $list_id ) . '/interest-categories/' . urlencode( $interest_category_id ) . '/interests/?count=100';
	$request_args = array(
		'headers' => array(
			'Authorization' => 'Basic ' . base64_encode( 'user:' . $api_key )
		)
	);

	$response = wp_remote_get( $request_url, $request_args );

	if ( is_wp_error( $response ) || 200 != wp_remote_retrieve_response_code( $response ) ) {
		return false;
	}

	$response_body = json_decode( wp_remote_retrieve_body( $response ), true );

	$interests = array();

	if ( ! empty( $response_body['interests'] ) && is_array( $response_body['interests'] ) ) {
		foreach ( $response_body['interests'] as $interest ) {
			$interests[ $interest['id'] ] = $interest['name'];
		}
	}

	return $interests;

}

/**
 * Adds a note to the customer when subscribing them to a MailChimp list fails.
 *
 * @param string         $list_id      ID of the list the user was being subscribed to.
 * @param int            $user_id      ID of the user we were trying to subscribe.
 * @param WP_Error|array $response     API request response.
 * @param array          $request_body Body we sent in the API request.
 *
 * @since 1.4.6
 * @return void
 */
function rcp_mailchimp_pro_add_customer_note_on_failure( $list_id, $user_id, $response, $request_body ) {

	if ( ! function_exists( 'rcp_get_customer_by_user_id' ) ) {
		return;
	}

	$customer = rcp_get_customer_by_user_id( $user_id );

	if ( ! $customer instanceof RCP_Customer ) {
		return;
	}

	$customer->add_note( sprintf( __( 'MailChimp Pro: Failed to subscribe customer to list %s. Response code: %d; Response: %s', 'rcp-mailchimp-pro' ), $list_id, wp_remote_retrieve_response_code( $response ), wp_remote_retrieve_body( $response ) ) );

}

add_action( 'rcp_mailchimp_pro_subscribe_failed', 'rcp_mailchimp_pro_add_customer_note_on_failure', 10, 4 );

/**
 * Adds a note to the customer when they are successfully subscribed to a MailChimp list.
 *
 * @param string $list_id ID of the list the user was subscribed to.
 * @param int    $user_id ID of the user who was subscribed.
 *
 * @since 1.4.6
 * @return void
 */
function rcp_mailchimp_pro_add_customer_note_on_success( $list_id, $user_id ) {

	if ( ! function_exists( 'rcp_get_customer_by_user_id' ) ) {
		return;
	}

	$customer = rcp_get_customer_by_user_id( $user_id );

	if ( ! $customer instanceof RCP_Customer ) {
		return;
	}

	$customer->add_note( sprintf( __( 'MailChimp Pro: Successfully subscribed customer to list %s.', 'rcp-mailchimp-pro' ), $list_id ) );

}

add_action( 'rcp_mailchimp_pro_subscribe_success', 'rcp_mailchimp_pro_add_customer_note_on_success', 10, 2 );